uses
  Crt, Dos, Graph;

var
  GraphDriver : integer;  { The Graphics device driver }
  GraphMode   : integer;  { The Graphics mode value }
  MaxX, MaxY  : word;     { The maximum resolution of the screen }
  ErrorCode   : integer;  { Reports any graphics errors }
  MaxColor    : word;     { The maximum color value available }
  OldExitProc : Pointer;  { Saves exit procedure address }
  col,row:integer;
  thefile: file of byte;
  ch:char;
  imm:array[1..16000] of byte;
  count,count2:longint;

{$F+}
procedure MyExitProc;
begin
  ExitProc := OldExitProc; { Restore exit procedure address }
  CloseGraph;              { Shut down the graphics system }
end; { MyExitProc }
{$F-}

procedure Initialize;
{ Initialize graphics and report any errors that may occur }
var
  InGraphicsMode : boolean; { Flags initialization of graphics mode }
  PathToDriver   : string;  { Stores the DOS path to *.BGI & *.CHR }
begin
  { when using Crt and graphics, turn off Crt's memory-mapped writes }
  DirectVideo := False;
  OldExitProc := ExitProc;                { save previous exit proc }
  ExitProc := @MyExitProc;                { insert our exit proc in chain }
  PathToDriver := '.';
  repeat

{$IFDEF Use8514}                          { check for Use8514 $DEFINE }
    GraphDriver := IBM8514;
    GraphMode := IBM8514Hi;
{$ELSE}
    GraphDriver := Detect;                { use autodetection }
{$ENDIF}

    InitGraph(GraphDriver, GraphMode, PathToDriver);
    ErrorCode := GraphResult;             { preserve error return }
    if ErrorCode <> grOK then             { error? }
    begin
      Writeln('Graphics error: ', GraphErrorMsg(ErrorCode));
      if ErrorCode = grFileNotFound then  { Can't find driver file }
      begin
        Writeln('Enter full path to BGI driver or type <Ctrl-Break> to quit:');
        Readln(PathToDriver);
halt(1);
        Writeln;
      end
      else
        Halt(1);                          { Some other error: terminate }
    end;
  until ErrorCode = grOK;
  Randomize;                { init random number generator }
  MaxColor := GetMaxColor;  { Get the maximum allowable drawing color }
  MaxX := GetMaxX;          { Get screen resolution values }
  MaxY := GetMaxY;
end; { Initialize }


procedure CrtModePlay;
{ Demonstrate the use of RestoreCrtMode and SetGraphMode }
var
  ViewInfo : ViewPortType;
  Ch       : char;
begin
  GetViewSettings(ViewInfo);
  SetTextJustify(CenterText, CenterText);
  with ViewInfo do
  begin
    RestoreCrtmode;
    Writeln('Now you are in text mode.');
  end;
end; { CrtModePlay }

begin { program body }
  Initialize;
  CrtModePlay;
assign(thefile,'imm.bmp');
reset(thefile);
writeln('loading....');
while (not eof(thefile) or keypressed) do
 begin
{skip first 62d bytes}
 count:=0;
 repeat;
  read(thefile,imm[count]);
  count:=count+1;
 until (count=63);

{store 16000 bytes (image data) into imm[] array}
 count:=1;
 repeat;
  read(thefile,imm[count]);
  count:=count+1;
 until (count=16000);

{first byte of data is LOWER LEFT corner of the image;
last byte of data is UPPER RIGHT corner of the image;
following instructions display image by writing dirctly
to video ram ($b800:0000), starting from the LOWEST line
of the screen (from $b800:$3e30 to $b800:$3e80 , line is $50 long.)
}
 row:=200;   {Begin from last line, first column}
 col:=1;
 count2:=1;  {index of imm[] array (image data)}
 count:=80*(row-1)+col-1;  {Memory offset to write into}

repeat; 
  mem[$b800:count]:=not imm[count2];
  col:=col+1;
  if col>80 then
{each line is $50 (=80d) long}
   begin
   col:=1;
   row:=row-1;
   end;
  count2:=count2+1;
  count:=80*(row-1)+col-1;
  until ((count=1) or keypressed or (count2=16000));
 end;
   ch:=readkey;
end.
